#
# MicroTESK PowerPC Edition
#
# Copyright (c) 2016 Institute for System Programming of the Russian Academy of Sciences
# All Rights Reserved
#
# Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
# 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
# http://www.ispras.ru
#

require_relative 'powerpc_base'

#
# Description:
#
# This small tests for ALU instructions.
#

class InstructionALU < PowerPCBaseTemplate

  def run
    trace "Run ALU instruction:"

    trace "\nPowerPC Arithmetic:\n"

    addi r1, r0, 0x11
    trace "(add): r1 = %x", gpr_observer(1)
    add r2, r1, r1
    trace "(add): r2 = %x", gpr_observer(2)
    addi r3, r0, 0x1
    trace "(add): r3 = %x", gpr_observer(3)

    ori r4, r2, 0x11
    trace "(ori): r4 = %x", gpr_observer(4)
    add_d r5, r4, r4
    trace "(add.): r5 = %x", gpr_observer(5)
    addo r2, r2, r2
    trace "(addo): r2 = %x", gpr_observer(2)
    addo_d r2, r2, r1
    trace "(addo.): r2 = %x", gpr_observer(2)

    subf r6, r1, r5
    trace "(sub): r6 = %x", gpr_observer(6)
    subf_d r7, r5, r1
    trace "(sub.): r7 = %x", gpr_observer(7)
    subf r8, r7, r4
    trace "(sub): r8 = %x", gpr_observer(8)
    subfo r9, r6, r8
    trace "(subo): r9 = %x", gpr_observer(9)
    subfo_d r10, r6, r9
    trace "(subo.): r10 = %x", gpr_observer(10)
    
    neg r11, r10
    trace "(neg): r11 = %x", gpr_observer(11)
    neg_d r11, r11
    trace "(neg.): r11 = %x", gpr_observer(11)
    addi r11, r0, 0x0
    trace "(add): r11 = %x", gpr_observer(11)
    nego r11, r11
    trace "(nego): r11 = %x", gpr_observer(11)
    nego_d r12, r9
    trace "(nego.): r12 = %x", gpr_observer(12)
    
    xor r13, r12, r10
    trace "(xor): r13 = %x", gpr_observer(13)
    xor_d r13, r13, r11
    trace "(xor.): r13 = %x", gpr_observer(13)
    
    rlwinm r14, r13, 31, 16, 15
    trace "(rlwinm): r14 = %x", gpr_observer(14)
    srwi r14, r13, 1
    trace "(srwi): r14 = %x", gpr_observer(14)
    srwi r14, r13, 2
    trace "(srwi): r14 = %x", gpr_observer(14)
    srwi_d r14, r10, 16
    trace "(srwi.): r14 = %x", gpr_observer(14)
    slwi r14, r10, 16
    trace "(slwi): r14 = %x", gpr_observer(14)
    slwi_d r14, r13, 1
    trace "(slwi.): r14 = %x", gpr_observer(14)
    extlwi r14, r10, 10, 5 
    trace "(extlwi): r14 = %x", gpr_observer(14)
    extlwi_d r14, r10, 15, 25 
    trace "(extlwi.): r14 = %x", gpr_observer(14)
    extrwi r14, r10, 10, 5 
    trace "(extrwi): r14 = %x", gpr_observer(14)
    extrwi_d r14, r10, 15, 25 
    trace "(extrwi.): r14 = %x", gpr_observer(14)
    rotlwi r14, r10, 8
    trace "(rotlwi): r14 = %x", gpr_observer(14)
    rotlwi_d r14, r10, 28
    trace "(rotlwi.): r14 = %x", gpr_observer(14)
    rotrwi r14, r10, 8
    trace "(rotrwi): r14 = %x", gpr_observer(14)
    rotrwi_d r14, r10, 28
    trace "(rotrwi.): r14 = %x", gpr_observer(14)
    clrlwi r14, r10, 8
    trace "(clrlwi): r14 = %x", gpr_observer(14)
    clrlwi_d r14, r10, 28
    trace "(clrlwi.): r14 = %x", gpr_observer(14)
    clrrwi r14, r10, 8
    trace "(clrrwi): r14 = %x", gpr_observer(14)
    clrrwi_d r14, r10, 28
    trace "(clrrwi.): r14 = %x", gpr_observer(14)
    clrlslwi r14, r10, 16, 8 
    trace "(clrlslwi): r14 = %x", gpr_observer(14)
    clrlslwi_d r14, r10, 24, 12 
    trace "(clrlslwi.): r14 = %x", gpr_observer(14)
    
    OR r15, r13, r8
    trace "(or): r15 = %x", gpr_observer(15)
    or_d r15, r13, r8
    trace "(or.): r15 = %x", gpr_observer(15)
    mr r15, r12
    trace "(mr): r15 = %x", gpr_observer(15)
    mr_d r15, r14
    trace "(mr.): r15 = %x", gpr_observer(15)
    srawi r15, r12, 16
    trace "(srawi): r15 = %x", gpr_observer(15)
    srawi_d r15, r13, 1
    trace "(srawi.): r15 = %x", gpr_observer(15)
    
    nop
    nop
  end

end
